import os
from django.contrib.staticfiles import finders
from django import template

register = template.Library()


@register.filter
def css(path: str) -> str:
    """
    Returns the content of a CSS file or all CSS files in a directory.
    """

    if not (path.endswith('.css') or os.path.isdir(finders.find(path))):
        raise ValueError(
            'The path must end with ".css" or '
            'be a directory containing CSS files.'
        )

    absolute_path = finders.find(path)

    if not absolute_path:
        raise FileNotFoundError(f"Static file or directory '{path}' not found.")

    if os.path.isdir(absolute_path):
        return _get_dir_css_content(absolute_path)

    # If the path is a file, return its content.
    with open(absolute_path, 'r') as file:
        return file.read()


def _get_dir_css_content(dir_path: str) -> str:
    css_content = ''
    for root, _, files in os.walk(dir_path):
        for file in sorted(files):  # Sort files alphabetically
            if file.endswith('.css'):
                file_path = os.path.join(root, file)
                with open(file_path, 'r') as css_file:
                    css_content += css_file.read() + '\n'

    return css_content



from unittest.mock import patch, mock_open as open_mock
from django.test import SimpleTestCase

from django_pdf_view.templatetags.css import css


class TestCssTemplateFilter(SimpleTestCase):

    def setUp(self):
        super().setUp()

        self.finders_find_patcher = patch(
            target='django_pdf_view.templatetags.css.finders.find'
        )
        self.mock_finders_find = self.finders_find_patcher.start()

    def test_css_file_found(self):
        self.mock_finders_find.return_value = '/path/to/static/file.css'
        mocked_file_content = 'body { background-color: #fff; }'
        with patch(
            target='builtins.open',
            new=open_mock(read_data=mocked_file_content)
        ) as mock_file:
            result = css('file.css')
            mock_file.assert_called_once_with('/path/to/static/file.css', 'r')
            self.assertEqual(result, mocked_file_content)

    @patch(
        target='builtins.open',
        new_callable=open_mock,
        read_data='body { color: black; }'
    )
    @patch(target='os.walk')
    @patch(target='os.path.isdir', return_value=True)
    def test_get_dir_css_content(self, _, mock_walk, mock_open):
        mock_walk.return_value = [
            ('/fake_dir', ('subdir',),
             ('style1.css', 'style2.css', 'not_a_css.txt')),
            ('/fake_dir/subdir', (), ('sub_style.css',))
        ]

        expected_content = (
            'body { color: black; }\n'
            'body { color: black; }\n'
            'body { color: black; }\n'
        )

        result = css('/fake_dir')

        self.assertEqual(result, expected_content)

        mock_open.assert_any_call('/fake_dir/style1.css', 'r')
        mock_open.assert_any_call('/fake_dir/style2.css', 'r')
        mock_open.assert_any_call('/fake_dir/subdir/sub_style.css', 'r')

        self.assertEqual(mock_open.call_count, 3)

    def test_css_file_not_found(self):
        self.mock_finders_find.return_value = None
        with self.assertRaises(FileNotFoundError):
            css('nonexistent.css')

    def test_css_invalid_extension(self):
        with self.assertRaises(ValueError):
            css('file.txt')

    def tearDown(self):
        super().tearDown()
        self.finders_find_patcher.stop()



from django.test import TestCase
from django.utils import translation

from django_pdf_view.services import create_pdf


class TestCreatePDF(TestCase):

    def test_create_pdf_with_minimal_arguments(self):
        pdf = create_pdf(
            template_name='django_pdf_view/examples/single_page.html'
        )

        self.assertEqual(pdf.template_name, 'django_pdf_view/pdf.html')
        self.assertEqual(pdf.get_title(), 'pdf')
        self.assertEqual(pdf.get_filename(), 'pdf.pdf')
        self.assertEqual(pdf.language, translation.get_language())

        self.assertEqual(len(pdf.pages), 1)
        page = pdf.pages[0]
        self.assertEqual(
            page.template_name,
            'django_pdf_view/examples/single_page.html'
        )
        self.assertEqual(page.number, 1)
        self.assertEqual(page.get_context(), {'page_number': 1})
        self.assertTrue(page.with_wrapper_html)

    def test_create_pdf_with_all_arguments(self):
        pdf = create_pdf(
            template_name='django_pdf_view/examples/single_page.html',
            title='My PDF',
            filename='my_pdf.pdf',
            context={'key': 'value'},
            language='rs',
        )

        self.assertEqual(pdf.template_name, 'django_pdf_view/pdf.html')
        self.assertEqual(pdf.get_title(), 'My PDF')
        self.assertEqual(pdf.get_filename(), 'my_pdf.pdf')
        self.assertEqual(pdf.language, 'rs')

        self.assertEqual(len(pdf.pages), 1)
        page = pdf.pages[0]
        self.assertEqual(
            page.template_name,
            'django_pdf_view/examples/single_page.html'
        )
        self.assertEqual(page.number, 1)
        self.assertEqual(page.get_context(), {
            'key': 'value',
            'page_number': 1,
        })
        self.assertTrue(page.with_wrapper_html)



from django.test import TestCase, RequestFactory

from django_pdf_view.pdf import PDF
from django_pdf_view.views import PDFView


class ConcretePDFView(PDFView):
    sample_pdf = PDF(
        template_name='django_pdf_view/pdf.html',
        language='en',
        filename='test.pdf',
        title='Test PDF'
    )

    def create_pdf(self):
        return self.sample_pdf


class TestPDFView(TestCase):

    @classmethod
    def setUpTestData(cls):
        cls.request_factory = RequestFactory()

    def test_create_pdf(self):
        view = ConcretePDFView()
        pdf = view.create_pdf()
        self.assertEqual(pdf, ConcretePDFView.sample_pdf)

    def test_html_response(self):
        view = ConcretePDFView()
        response = view.html_response()
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response['Content-Type'], 'text/html')

    def test_pdf_response(self):
        view = ConcretePDFView()
        response = view.pdf_response()
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response['Content-Type'], 'application/pdf')
        self.assertEqual(
            response['Content-Disposition'],
            f'inline; filename="{ConcretePDFView.sample_pdf.filename}"'
        )

    def test_download_pdf_response(self):
        view = ConcretePDFView()
        response = view.download_pdf_response()
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response['Content-Type'], 'application/pdf')
        self.assertEqual(
            response['Content-Disposition'],
            f'attachment; filename="{ConcretePDFView.sample_pdf.filename}"'
        )

    def test_get_method_with_html_response(self):
        # When response type is not provided through view initialization,
        # it should be determined from the URL query parameters.
        request = self.request_factory.get('/?html=true')
        response = ConcretePDFView.as_view()(request)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response['Content-Type'], 'text/html')

        # When response type is provided through view initialization,
        # it should override the URL query parameters.
        request = self.request_factory.get('/?download=true')
        response = ConcretePDFView.as_view(response_type='html')(request)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response['Content-Type'], 'text/html')

    def test_get_method_with_pdf_response(self):
        # When response type is not provided through view initialization,
        # and it is not provided through URL query parameters,
        # it should default to 'pdf'.
        request = self.request_factory.get('/')
        response = ConcretePDFView.as_view()(request)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response['Content-Type'], 'application/pdf')
        self.assertEqual(
            response['Content-Disposition'],
            f'inline; filename="{ConcretePDFView.sample_pdf.filename}"'
        )

        # When response type is provided through view initialization,
        # it should override the URL query parameters.
        request = self.request_factory.get('/?html=true')
        response = ConcretePDFView.as_view(response_type='pdf')(request)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response['Content-Type'], 'application/pdf')
        self.assertEqual(
            response['Content-Disposition'],
            f'inline; filename="{ConcretePDFView.sample_pdf.filename}"'
        )

    def test_get_method_with_download_pdf_response(self):
        # When response type is not provided through view initialization,
        # it should be determined from the URL query parameters.
        request = self.request_factory.get('/?download=true')
        response = ConcretePDFView.as_view()(request)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response['Content-Type'], 'application/pdf')
        self.assertEqual(
            response['Content-Disposition'],
            f'attachment; filename="{ConcretePDFView.sample_pdf.filename}"'
        )

        # When response type is provided through view initialization,
        # it should override the URL query parameters.
        request = self.request_factory.get('/?html=true')
        response = ConcretePDFView.as_view(response_type='download')(request)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response['Content-Type'], 'application/pdf')
        self.assertEqual(
            response['Content-Disposition'],
            f'attachment; filename="{ConcretePDFView.sample_pdf.filename}"'
        )



import os
from django.test import TestCase

from django_pdf_view.decorators import with_tmp_env_var


class TestWithTmpEnvVar(TestCase):

    def test_existing_env_var_overridden_during_function_execution(self):
        os.environ['TEST_ENV_VAR'] = 'original_value'

        @with_tmp_env_var('TEST_ENV_VAR', 'temp_value')
        def test_function():
            self.assertEqual(os.environ['TEST_ENV_VAR'], 'temp_value')

        test_function()

        self.assertEqual(os.environ['TEST_ENV_VAR'], 'original_value')

    def test_new_env_var_set_during_function_execution(self):
        if 'TEST_ENV_VAR' in os.environ:
            del os.environ['TEST_ENV_VAR']

        @with_tmp_env_var('TEST_ENV_VAR', 'temp_value')
        def test_function():
            self.assertEqual(os.environ['TEST_ENV_VAR'], 'temp_value')

        test_function()

        self.assertNotIn('TEST_ENV_VAR', os.environ)



import os
from contextlib import contextmanager


@contextmanager
def tmp_env_var(key: str, value: str):
    original_value = os.environ.get(key)
    os.environ[key] = value
    try:
        yield
    finally:
        if original_value is not None:
            os.environ[key] = original_value
        else:
            del os.environ[key]



from django.contrib.staticfiles import finders
from django import template
from django.utils.safestring import mark_safe

register = template.Library()


@register.simple_tag
def svg(filepath: str) -> str:
    """
    Returns the content of an SVG file.
    """

    if not filepath.endswith('.svg'):
        raise ValueError('The file path must end with ".svg"')

    if absolute_path := finders.find(filepath):
        with open(absolute_path, 'r') as file:
            return mark_safe(file.read())

    raise FileNotFoundError(f'Static file "{filepath}" not found.')



from unittest import TestCase
from unittest.mock import patch

from django_pdf_view.pdf_page import PDFPage


class TestPDFPage(TestCase):

    def test_initialization(self):
        # Without context:
        pdf_page = PDFPage(
            template_name='pdf_page.html',
            number=1,
        )
        self.assertEqual(pdf_page.template_name, 'pdf_page.html')
        self.assertEqual(pdf_page.number, 1)
        self.assertEqual(pdf_page.context, {})

        # With context:
        pdf_page = PDFPage(
            template_name='pdf_page.html',
            number=1,
            context={'foo': 'bar'}
        )
        self.assertEqual(pdf_page.template_name, 'pdf_page.html')
        self.assertEqual(pdf_page.number, 1)
        self.assertEqual(pdf_page.context, {'foo': 'bar'})

    @patch('django_pdf_view.pdf_page.render_to_string')
    def test_render_html_with_wrapper_html(self, mock_render_pdf_to_string):
        """
        If with_wrapper_html is True (default), render_html should
        return the rendered template wrapped in a div with the class
        'page' and 'page--{page_number}'.
        """
        mock_render_pdf_to_string.return_value = '<html>PDF Page</html>'
        pdf_page = PDFPage(
            template_name='pdf_page.html',
            number=1,
            context={'foo': 'bar'}
        )
        html = pdf_page.render_html()
        expected_html = (
            '<div class="page page--1">'
            '<html>PDF Page</html>'
            '</div>'
        )
        self.assertEqual(html, expected_html)

    @patch('django_pdf_view.pdf_page.render_to_string')
    def test_render_html_without_wrapper_html(self, mock_render_pdf_to_string):
        """
        If with_wrapper_html is False, render_html should return the
        rendered template without any wrapping.
        """
        mock_render_pdf_to_string.return_value = '<html>PDF Page</html>'
        pdf_page = PDFPage(
            template_name='pdf_page.html',
            number=1,
            context={'foo': 'bar'},
            with_wrapper_html=False,
        )
        html = pdf_page.render_html()
        self.assertEqual(html, mock_render_pdf_to_string.return_value)

    def test_get_context(self):
        pdf_page = PDFPage(
            template_name='pdf_page.html',
            number=1,
            context={'foo': 'bar'}
        )
        context = pdf_page.get_context(bar='baz')
        self.assertEqual(context, {
            'foo': 'bar',
            'page_number': 1,
            'bar': 'baz',
        })



from io import BytesIO

from django.template.loader import render_to_string
from django.utils import translation
from pdfkit import from_string

from django_pdf_view.pdf_page import PDFPage
from django_pdf_view.decorators import override_language


class PDF:
    page_class = PDFPage
    default_template_name = 'django_pdf_view/pdf.html'

    def __init__(
        self,
        template_name: str = default_template_name,
        language: str = None,
        filename: str = None,
        title: str = None,
    ):
        self.template_name = template_name
        self.language = language or translation.get_language()
        self._filename = filename
        self._title = title
        self._pages = []
        self._in_memory_pdf = None

    def add_page(
        self,
        template_name: str,
        context: dict = None,
        with_wrapper_html: bool = True,
    ) -> None:
        page_number = len(self._pages) + 1
        new_page = self.page_class(
            template_name=template_name,
            number=page_number,
            context=context,
            with_wrapper_html=with_wrapper_html,
        )
        self._pages.append(new_page)

    @property
    def pages(self) -> list[PDFPage]:
        self._pages.sort(key=lambda page: page.number)
        return self._pages

    @property
    def filename(self) -> str:
        return self.get_filename()

    @override_language
    def get_filename(self) -> str:
        if self._filename:
            return self._filename
        return self.template_name.split('/')[-1].replace('.html', '.pdf')

    @override_language
    def render_html(self) -> str:
        return render_to_string(
            template_name=self.template_name,
            context=self.get_context()
        )

    def get_context(self) -> dict:
        total_pages = len(self._pages)
        pages_html = ''.join([
            page.render_html(total_pages=total_pages) for page in self.pages
        ])
        return {
            'pages_html': pages_html,
            'title': self.get_title(),
        }

    def get_title(self) -> str:
        title = self._title or self.filename
        if title.endswith('.pdf'):
            title = title[:-4]
        return title

    @property
    def in_memory_pdf(self) -> BytesIO:
        if not self._in_memory_pdf:
            html = from_string(
                input=self.render_html(),
                options={
                    'page-size': 'A4',
                    'margin-top': '0mm',
                    'margin-right': '0mm',
                    'margin-bottom': '0mm',
                    'margin-left': '0mm',
                    'dpi': '300',
                }
            )
            self._in_memory_pdf = BytesIO(html)
        return self._in_memory_pdf



from django.template.loader import render_to_string


class PDFPage:

    def __init__(
        self,
        template_name: str,
        number: int,
        context: dict = None,
        with_wrapper_html: bool = True,
    ):
        self.template_name = template_name
        self.number = number
        self.context = context or {}
        self.with_wrapper_html = with_wrapper_html

    def render_html(self, **extra_context) -> str:
        html = render_to_string(
            template_name=self.template_name,
            context=self.get_context(**extra_context)
        )
        if self.with_wrapper_html:
            return (
                f'<div class="page page--{self.number}">'
                f'{html}'
                '</div>'
            )
        return html

    def get_context(self, **extra_context) -> dict:
        return {
            **self.context,
            'page_number': self.number,
            **extra_context,
        }



from django_pdf_view.pdf import PDF


def create_pdf(
    template_name: str,
    title: str = None,
    filename: str = None,
    context: dict = None,
    language: str = None,
) -> PDF:
    """
    Creates a PDF object with a single page.
    """
    pdf = PDF(
        title=title,
        filename=filename,
        language=language
    )
    pdf.add_page(template_name=template_name, context=context)

    return pdf



from abc import abstractmethod
from typing import Literal

from django.http import HttpResponse, FileResponse
from django.views import View

from django_pdf_view.decorators import with_tmp_env_var
from django_pdf_view.pdf import PDF


class PDFView(View):
    ResponseType = Literal['pdf', 'html', 'download']
    response_type: ResponseType | None = None

    @classmethod
    def as_view(cls, response_type: ResponseType = None, **initkwargs):
        return super().as_view(response_type=response_type, **initkwargs)

    def __init__(self, *args, response_type: ResponseType = None, **kwargs):
        super().__init__(*args, **kwargs)
        self.response_type = response_type

    def setup(self, request, *args, **kwargs):
        super().setup(request, *args, **kwargs)
        if self.response_type:
            pass
        elif self.request.GET.get('html') == 'true':
            self.response_type = 'html'
        elif self.request.GET.get('download') == 'true':
            self.response_type = 'download'
        else:
            self.response_type = 'pdf'

    def get(self, *args, **kwargs):
        if self.response_type == 'html':
            return self.html_response()

        if self.response_type == 'download':
            return self.download_pdf_response()

        return self.pdf_response()

    @with_tmp_env_var('QT_QPA_PLATFORM', 'offscreen')
    def pdf_response(self) -> FileResponse:
        """
        This response will display the PDF in the
        browser without downloading it.
        """
        pdf = self.create_pdf()
        return FileResponse(pdf.in_memory_pdf, filename=pdf.filename)

    @with_tmp_env_var('QT_QPA_PLATFORM', 'offscreen')
    def download_pdf_response(self) -> FileResponse:
        """
        This response will download the PDF without
        displaying it in the browser.
        """
        pdf = self.create_pdf()
        response = FileResponse(pdf.in_memory_pdf, filename=pdf.filename)
        content_disposition = f'attachment; filename="{pdf.filename}"'
        response['Content-Disposition'] = content_disposition
        return response

    def html_response(self) -> HttpResponse:
        """
        This response will display the HTML in the
        browser.
        """
        pdf = self.create_pdf()
        return HttpResponse(
            content=pdf.render_html(),
            content_type='text/html',
        )

    @abstractmethod
    def create_pdf(self) -> PDF:
        pass  # pragma: no cover



import os

from django.test import TestCase

from django_pdf_view.context_managers import tmp_env_var


class TestTmpEnvVar(TestCase):

    def test_set_new_env_var(self):
        key = 'TEST_ENV_VAR'
        value = 'test_value'

        if key in os.environ:
            del os.environ[key]

        with tmp_env_var(key, value):
            self.assertEqual(os.environ[key], value)

        self.assertNotIn(key, os.environ)

    def test_override_existing_env_var(self):
        key = 'TEST_ENV_VAR'
        original_value = 'original_value'
        new_value = 'new_value'

        os.environ[key] = original_value

        with tmp_env_var(key, new_value):
            self.assertEqual(os.environ[key], new_value)

        self.assertEqual(os.environ[key], original_value)



from django_pdf_view.pdf import PDF
from django_pdf_view.services import create_pdf
from django_pdf_view.views.pdf_view import PDFView


class SinglePageExamplePDFView(PDFView):

    def create_pdf(self) -> PDF:
        return create_pdf(
            template_name='django_pdf_view/examples/single_page.html',
            title='Single Page Example PDF',
            filename='single_page_example_pdf.pdf',
        )


class MultiPageExamplePDFView(PDFView):

    def create_pdf(self) -> PDF:
        pdf = PDF(
            title='Multi Page Example PDF',
            filename='multi_page_example_pdf.pdf',
        )

        pdf.add_page(template_name='django_pdf_view/examples/multi_page_1.html')
        pdf.add_page(template_name='django_pdf_view/examples/multi_page_2.html')
        pdf.add_page(template_name='django_pdf_view/examples/multi_page_3.html')

        return pdf



from functools import wraps
from typing import Callable, Any
from django.utils import translation

from django_pdf_view.context_managers import tmp_env_var


def override_language(method: Callable) -> Callable:
    """
    Decorator that overrides the language of the current thread
    with the language of the instance of the class where the
    decorated method is called.
    """

    @wraps(method)
    def wrapper(self: Any, *args: Any, **kwargs: Any) -> Any:
        language = self.language or translation.get_language()
        with translation.override(language):
            return method(self, *args, **kwargs)

    return wrapper


def with_tmp_env_var(key: str, value: str) -> Callable:
    """
    Decorator that sets a temporary environment variable
    with the given key and value for the duration of the
    decorated function.
    """

    def decorator(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            with tmp_env_var(key, value):
                return func(*args, **kwargs)

        return wrapper

    return decorator



from django.urls import path, include

from django_pdf_view.views import (
    SinglePageExamplePDFView as SPView,
    MultiPageExamplePDFView as MPView,
)

app_name = 'django-pdf-view'

single_page_urlpatterns = [
    path('pdf/', SPView.as_view(response_type='pdf'), name='pdf'),
    path('html/', SPView.as_view(response_type='html'), name='html'),
    path(
        'download/',
        SPView.as_view(response_type='download'),
        name='download'
    ),
]

multi_page_urlpatterns = [
    path('pdf/', MPView.as_view(response_type='pdf'), name='pdf'),
    path('html/', MPView.as_view(response_type='html'), name='html'),
    path(
        'download/',
        MPView.as_view(response_type='download'),
        name='download'
    ),
]

urlpatterns = [
    path(
        'single-page/',
        include(
            (single_page_urlpatterns, 'django_pdf_view'),
            namespace='single_page'
        ),
    ),
    path(
        'multi-page',
        include(
            (multi_page_urlpatterns, 'django_pdf_view'),
            namespace='multi_page'
        ),
    ),
]



from django.http import FileResponse
from django.test import TestCase
from django.urls import reverse

from django_pdf_view.views import MultiPageExamplePDFView


class TestMultiPageExamplePDFView(TestCase):

    def test_create_pdf(self):
        view = MultiPageExamplePDFView()
        pdf = view.create_pdf()

        self.assertEqual(pdf.get_title(), 'Multi Page Example PDF')
        self.assertEqual(pdf.get_filename(), 'multi_page_example_pdf.pdf')
        self.assertEqual(pdf.template_name, 'django_pdf_view/pdf.html')
        self.assertEqual(len(pdf.pages), 3)
        for page in pdf.pages:
            self.assertEqual(
                page.template_name,
                f'django_pdf_view/examples/multi_page_{page.number}.html'
            )

    def test_get_pdf(self):
        url_path = reverse('examples:multi_page:pdf')
        response = self.client.get(path=url_path)
        self.assertEqual(response.status_code, 200)
        self.assertIsInstance(response, FileResponse)

    def test_get_html(self):
        url_path = reverse('examples:multi_page:html')
        response = self.client.get(path=url_path)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response['Content-Type'], 'text/html')

    def test_download_pdf(self):
        url_path = reverse('examples:multi_page:download')
        response = self.client.get(path=url_path)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response['Content-Type'], 'application/pdf')
        self.assertEqual(
            response['Content-Disposition'],
            'attachment; filename="multi_page_example_pdf.pdf"'
        )



from unittest import TestCase
from unittest.mock import patch
from io import BytesIO

from django_pdf_view.pdf import PDF
from django_pdf_view.pdf_page import PDFPage


class TestPDF(TestCase):

    def setUp(self):
        super().setUp()
        self.render_page_html_patcher = patch(
            target='django_pdf_view.pdf_page.PDFPage.render_html',
            return_value='<div>PDF Page</div>'
        )
        self.mock_render_page_html = self.render_page_html_patcher.start()

    def test_initialization(self):
        pdf = PDF(
            template_name='pdf.html',
            language='en',
            filename='test.pdf',
            title='Test PDF'
        )
        self.assertEqual(pdf.template_name, 'pdf.html')
        self.assertEqual(pdf.language, 'en')
        self.assertEqual(pdf.filename, 'test.pdf')
        self.assertEqual(pdf._title, 'Test PDF')
        self.assertEqual(pdf.pages, [])

    def test_add_page(self):
        pdf = PDF(template_name='pdf.html')
        pdf.add_page(template_name='pdf_page.html')
        self.assertEqual(len(pdf.pages), 1)
        self.assertIsInstance(pdf.pages[0], PDFPage)
        self.assertEqual(pdf.pages[0].template_name, 'pdf_page.html')

    @patch('django_pdf_view.pdf.render_to_string')
    def test_render_html(self, mock_render_pdf_to_string):
        mock_render_pdf_to_string.return_value = '<html>PDF</html>'
        pdf = PDF(
            template_name='pdf.html',
            title='Awesome PDF'
        )
        pdf.add_page(template_name='pdf_page.html')
        html = pdf.render_html()
        mock_render_pdf_to_string.assert_called_once_with(
            template_name='pdf.html',
            context={
                'pages_html': '<div>PDF Page</div>',
                'title': 'Awesome PDF'
            }
        )
        self.mock_render_page_html.assert_called_once()
        self.assertEqual(html, '<html>PDF</html>')

    @patch('django_pdf_view.pdf.from_string')
    def test_in_memory_pdf(self, mock_from_string):
        mock_from_string.return_value = b'PDF content'
        pdf = PDF(template_name='django_pdf_view/pdf.html')
        pdf.add_page(template_name='django_pdf_view/pdf_page.html')
        in_memory_pdf = pdf.in_memory_pdf
        self.assertIsInstance(in_memory_pdf, BytesIO)
        self.assertEqual(in_memory_pdf.getvalue(), b'PDF content')

    def test_filename(self):
        """
        When the filename is provided, it should be returned.
        When the filename is not provided, the template name
        should be used to generate the filename.
        """
        named_pdf = PDF(template_name='pdf.html', filename='test.pdf')
        self.assertEqual(named_pdf.get_filename(), 'test.pdf')
        self.assertEqual(named_pdf.filename, 'test.pdf')

        nameless_pdf = PDF(template_name='document.html')
        self.assertEqual(nameless_pdf.get_filename(), 'document.pdf')
        self.assertEqual(nameless_pdf.filename, 'document.pdf')

    def test_get_title(self):
        """
        When the title is provided, it should be returned.
        When the title is not provided, the template name
        should be used to generate the title.
        """
        pdf = PDF(template_name='pdf.html', title='Custom Title')
        self.assertEqual(pdf.get_title(), 'Custom Title')

        pdf_without_title = PDF(template_name='document.html')
        self.assertEqual(pdf_without_title.get_title(), 'document')

    def test_get_context(self):
        pdf = PDF(template_name='pdf.html')
        pdf.add_page(template_name='pdf_page.html')
        context = pdf.get_context()
        self.assertEqual(context, {
            'pages_html': '<div>PDF Page</div>',
            'title': 'pdf'
        })

    def tearDown(self):
        super().tearDown()
        self.render_page_html_patcher.stop()



from django.test import TestCase
from unittest.mock import patch, mock_open

from django_pdf_view.templatetags.svg import svg


class TestSvgTemplateTag(TestCase):

    @patch(target='django_pdf_view.templatetags.svg.finders.find')
    def test_svg_file_found(self, mock_finders_find):
        mock_finders_find.return_value = '/path/to/static/file.svg'
        mocked_file_content = '<svg></svg>'
        with patch(
            target='builtins.open',
            new=mock_open(read_data=mocked_file_content)
        ) as mock_file:
            result = svg('file.svg')
            mock_file.assert_called_once_with('/path/to/static/file.svg', 'r')
            self.assertEqual(result, mocked_file_content)

    @patch(
        target='django_pdf_view.templatetags.svg.finders.find',
        return_value=None
    )
    def test_svg_file_not_found(self, _):
        with self.assertRaises(FileNotFoundError):
            svg('nonexistent.svg')

    def test_svg_invalid_extension(self):
        with self.assertRaises(ValueError):
            svg('file.txt')
